function varargout=P3R1F_keioGLM(fcn,varargin)
% POTATo : Research Mode 1st-Level-Analysis Function "keio GLM"
%
%  Beta value as Summary Statics 
%  from GLM analysis
%
% == History ==
%  2011.03.25 :
%  2011.06.25 :
%  2013.12.26 :

%##########################################################################
% Launcher
%##########################################################################
if nargin==0, fcn='help'; end

switch fcn
    case {'createBasicInfo','CreateGUI',...
      'Activate','Suspend',...
      'MakeExeData','SetExeData',...
      'execute',...
      'dofirst',...
      'CallBack_ppmModelKind',...
      'CallBack_ModelFileSelect','CallBack_CheckModel'}

    if nargout,
      [varargout{1:nargout}] = feval(fcn, varargin{:});
    else
      feval(fcn, varargin{:});
    end
  otherwise
    error('Not Implemented Function : %s',fcn);
end



%##########################################################################
function basic_info=createBasicInfo()
% Get Basic-Info of this function
%##########################################################################
basic_info.name = 'keio-lscp GLM';   % Display-Name 
basic_info.ver  = 1.0;          % Version (double)
basic_info.fcn  = mfilename;    % Function-Name



%##########################################################################
% GUI Control
%##########################################################################

%==========================================================================
function tag=mytag(tag)
% Tool : make unique tagname.
%==========================================================================
tag=[tag '_' mfilename];


%==========================================================================
function hs=CreateGUI(hs)
% Create Related GUI
%==========================================================================

%----------------------
% Set GUI Common Property.
%----------------------
c=get(hs.figure1,'Color');
prop={'Units','pixels','Visible','off'};
prop_t={prop{:},'style','text',...
  'HorizontalAlignment','left',...
  'BackgroundColor',c};

%----------------------
% Get GUI Position...
%----------------------
apidata={};

%p0=get(hs.frm_R1stExecution,'Position');
pt=get(hs.txt_R1st_ExecuteName,'Position');
pp=get(hs.pop_R1st_ExecuteName,'Position');

%----------------------
% Get Channel number...added by mu
%----------------------
filedata=get(hs.lbx_fileList,'UserData');
a=feval('DataDef2_Analysis','load',filedata(1));
key.actdata.data=a;
key.filterManage=a.data.filterdata;
fname=feval('DataDef2_Analysis','make_mfile',key);
[cdata]=scriptMeval(fname,'chdata');
popmenu_chN = size(cdata{1,1}.flag, 3);
popmenu_ch = cell(1,popmenu_chN);
for i = 1 : popmenu_chN
    popmenu_ch{1,i} = num2str(i);
end
clear cdata popmenu_chN;
try 
  [p f]=fileparts(fname);
  clear(f); 
catch
end
try delete(fname); catch  end

%----------------------
% Set GUI Individual Property.
%----------------------

%--- add 2011/06/25

% nhs1=[];
% nhs2=[];

%.. create Block Information GUI objects
dy=30;
dx=120;
mg=10;
p=[pt(1) pp(2)-mg*3 dx dy];
[hs ,dummy, p] = sub_MakeGUI_Editbox(hs, 'TaskDur', '15.0', 'Stimulus period (sec)', p, [0 0]);
p=[p(1)+dx+mg p(2) dx-20 dy];
[hs ,dummy, p] = sub_MakeGUI_Editbox(hs, 'PreStim', '2.0', 'Pre-stimlus (sec)', p, [0 0]);
p=[p(1)+(dx-20)+mg p(2) dx-20 dy];
[hs ,dummy, p] = sub_MakeGUI_Editbox(hs, 'PostStim', '10.0', 'Post-stimlus (sec)', p, [0 0]);

%.. create Regressor Information GUI object
dx = 250;
p=[pt(1) pp(2)-mg*7 dx dy];
[hs, myPPMhnd, p] = sub_MakeGUI_Popupmenu(hs, 'ModelKind', ...
    {'Boxcar', 'Model from File', 'Optimal Model from File made in Stat'}, ...
    1, 'Use Model', p, [0 0]);

% boxcar ------------------------------------------
newhs1 = [];
dx = 90;
p=[pt(1) pp(2)-mg*10 dx dy];
[hs newhs1(end+1:end+2) p] = sub_MakeGUI_Editbox(hs, 'BoxWidth', '1.0', 'Box width (sec)', p,[0 0]);
dx = 85;
p=[p(1)+90+mg p(2) dx dy];
[hs newhs1(end+1:end+2) p] = sub_MakeGUI_Editbox(hs, 'BoxNum', '20', 'Number of Box', p, [0 0]);
dx = 150;
p=[p(1)+85+mg p(2) dx dy];
[hs newhs1(end+1:end+2) p] = sub_MakeGUI_Editbox(hs, 'BoxOnset', '1.0', 'Box Onset Asynchrony (sec)', p,[0 0]);

% model from file & optimal model from file --------
newhs2 = [];
dx = 250;
p=[pt(1)+20+40 pp(2)-mg*10 dx dy];
[hs, newhs2(end+1:end+2), p] = sub_MakeGUI_Editbox(hs, 'ModelFileName', '', 'Model data file name', p, [0 0]);
cbstr = [mfilename '(''CallBack_ModelFileSelect'')'];
p=[pt(1)+20 p(2)-20 40 20];
[hs, newhs2(end+1), p] = sub_MakeGUI_Button(hs, 'ModelSelectBtn', cbstr, '...', p, [0 0]);


%... create Residual Regressor
dx = 250;
p=[pt(1) pp(2)-mg*14 dx dy];
[hs dymmy p] = sub_MakeGUI_Editbox(hs, 'sincosReg', '0.25 0.5 1 2 3 4 5', 'Sine & Cosine Regressors', p,[0 0]);
% [hs, myCBXhnd] = sub_MakeGUI_Checkbox(hs, 'ResidualReg', 0, 'Sine & Cosine regressor', p,[0 0]);


%.. creat Design Matrix Plot GUI object
dx=80;
p=[pt(1) pp(2)-mg*18 dx dy];
[hs, dummy, p] = sub_MakeGUI_Popupmenu(hs, 'Kind', {'Oxy','Deoxy','Total'}, 1, '(Hb Kind)', p, [0 0]);
p=[p(1)+mg+dx p(2) dx dy];
[hs, dummy, p] = sub_MakeGUI_Popupmenu(hs, 'SelectCh', popmenu_ch, 1, '(Plot Channel)', p, [0 0]);

sx=120;
p=[p(1)+4*mg+dx p(2)-mg*2 sx dy];
cbstr = [mfilename '(''CallBack_CheckModel'')'];
[hs] = sub_MakeGUI_Button(hs, 'CheckModel', cbstr, 'Check Design Matrix', p,[0 0]);

%Ōɐݒ肵ȂPopupmenuhsɍȂFȂH
UD.hndSet1 = newhs1;
UD.hndSet2 = newhs2;
cbstr=[mfilename '(''CallBack_ppmModelKind'',gco)'];
set(myPPMhnd,'UserData',UD, 'Callback', cbstr);
% set(myCBXhnd,'UserData',UD, 'Callback', cbstr);
% set(myCBXhnd,'UserData',hs, 'Callback', cbstr);
%--- add mu 2011/06/25

%   [cdata, chdata] = scriptMeval(varargin{2},'cdata','chdata');

% Save API Data
setappdata(hs.figure1,mfilename,apidata);




%==========================================================================
function [h, myhs]=myhandles(hs)
% My Handle List
%==========================================================================
fn=fieldnames(hs);
h=[];
for k=1:length(fn)
	if ~isempty(findstr(fn{k},mfilename))
		h(end+1)=hs.(fn{k});
		myhs.(fn{k}) = hs.(fn{k});
	end
end


%==========================================================================
function h=Activate(hs,sdt)
% My GUI Visible On
%==========================================================================
if sdt==3
  P3_gui_Research_1st('disable_execution',hs,...
    {'Error:','Continuous/Block Mixed'});
  return;
end

h=myhandles(hs);

% %##for debug###################
% try delete(h);catch;end
% hs=CreateGUI(hs);
% guidata(hs.figure1,hs);
% h=myhandles(hs);
% %#############################

set(h,'Visible','on');
% CallBack_cbxModelType(hs.(mytag('cbx_ModelType')));
CallBack_ppmModelKind(hs.(mytag('ppm_ModelKind')));

% Suspend API's
apidata=getappdata(hs.figure1,mfilename);
for ii=1:length(apidata)
  try
    [h,sdt]=feval(apidata{ii}.fcn,'Activate',hs,apidata{ii},sdt);
  catch
  end
end

%==========================================================================
function h=Suspend(hs)
% My GUI Visible Off
%==========================================================================
h=myhandles(hs);
set(h,'Visible','off');



%##########################################################################
% GUI Callbacks
%##########################################################################

%==========================================================================
function CallBack_ppmModelKind(hObject)
%==========================================================================
v=get(hObject,'Value');
hs=get(hObject,'UserData');
if v==1
	set(hs.hndSet1,'visible','on');
	set(hs.hndSet2,'visible','off');
else
	set(hs.hndSet1,'visible','off');
	set(hs.hndSet2,'visible','on');
end

%==========================================================================
function CallBack_ModelFileSelect
%==========================================================================
d=guidata(gcf);
while 1
	[f p]=uigetfile('*.mat','Select a model data file');
	if  isequal(f,0) || isequal(p,0), return;end
	try
		load([p f])
		figure;clf;
		plot(Model.HRF);
		break;
	catch
		errordlg('Can''t read Model data');
        break; %.. add 2011/06/21
	end
end
set(d.(mytag('edt_ModelFileName')), 'String',[p f]);


%==========================================================================
function CallBack_CheckModel
%==========================================================================
hs=guidata(gcf);
v=sub_Gather_Variables(hs);
% M=P3R2F_keioGLMtest('sub_MakeFIR_Model',v);

%---add mu
datar = P3_gui_Research_1st('makeSSReal0',hs,false);

tmpf = 1;
tmpch = v.plotCh;
tmphb = v.selectHbKind;

dur = 10 * (str2double(v.blkpre_sec) ...
    + str2double(v.taskDuration) + str2double(v.blkpost_sec));

R = [];

    
%--- Evalute m-file of preprocessor
key.actdata.data=datar.AnaFiles{tmpf};
key.filterManage=datar.AnaFiles{tmpf}.data.filterdata; %.. Filter
dfcn='DataDef2_Analysis'; % Data-Function
fname=feval(dfcn,'make_mfile',key);
try
[data,hdata] = scriptMeval(fname, 'data', 'hdata');
catch
end

stimkind = unique(hdata.stim(:,1));


%--- Make regressors
if v.selectedModelType == 1
    %------ select Boxcar
    M = P3R2F_keioGLMtest('sub_MakeFIR_Model', v);
    for tmpstim = 1 : length(stimkind)
        R = [R; P3P_addRegressor_ChModel(hdata,v,M,tmpch,stimkind(tmpstim))];
    end
elseif v.selectedModelType == 2
    %------ select Model from File
    load(v.modelFileName);
    M = Model.HRF;
    if size(M,1) > size(M,2); M = M'; end;
    for tmpstim = 1 : length(stimkind)
        R = [R; P3P_addRegressor_ChModel(hdata,v,M,tmpch,stimkind(tmpstim))];
    end
else
    %------ select Optimal Model from File made in Stat.
    load(v.modelFileName);
    M = Model.HRF;
    if dur < length(M)
        M = M(1:dur);
    end
    if size(M,1) > size(M,2); M = M'; end;
    timeReso = Model.timeReso;
    for tmpstim = 1 : length(stimkind)
        Tau = Model.Tau(tmpf,tmpch,1,tmpstim);
        if Tau > 0
%             M = [zeros(1,fix(Tau/timeReso)) M];
            shift = zeros(1, fix(Tau/timeReso));
            tmpM = [shift M(1:end-length(shift))];
        elseif Tau < 0
%             M = M(-fix(Tau/timeReso):end);
            shift = zeros(1, fix(-Tau/timeReso));
            tmpM = [M(length(shift)+1:end) shift];
        else
            tmpM = M;
        end
        R = [R; P3P_addRegressor_ChModel(hdata,v,tmpM,tmpch,stimkind(tmpstim))];
    end
end
%---- Residual Regressor
maxamp = max(max(abs(R)));
N = str2num(v.sincosReg);
R = [R; maxamp(1)*P3P_keioGLM_addRegressor_SinFluctuationNoise(hdata, N, tmpch)];
R = [R; maxamp(1)*P3P_keioGLM_addRegressor_1(hdata, tmpch)];
    
clear hdata data;

%---add mu

figure;
imagesc(R);
xlabel(sprintf('time (time point @%0.1fHz)',1000/v.samplingperiod));
ylabel('regressor number');
title('Design matrix')
colormap gray


%==========================================================================
function AD=sub_Gather_Variables(hs)
%==========================================================================
AD.blkpre_sec = get(hs.(mytag('edt_PreStim')),'String');
AD.blkpost_sec = get(hs.(mytag('edt_PostStim')),'String');
AD.taskDuration = get(hs.(mytag('edt_TaskDur')),'String');

AD.boxwidth_sec = get(hs.(mytag('edt_BoxWidth')),'String');
AD.boxshift_sec = get(hs.(mytag('edt_BoxOnset')),'String');
AD.regN = get(hs.(mytag('edt_BoxNum')),'String');

AD.modelFileName = get(hs.(mytag('edt_ModelFileName')),'String');

%--- add 2011/06/25
AD.selectedModelType = get(hs.(mytag('ppm_ModelKind')),'Value');
AD.sincosReg = get(hs.(mytag('edt_sincosReg')),'String');
AD.selectHbKind = get(hs.(mytag('ppm_Kind')), 'Value');
AD.plotCh = get(hs.(mytag('ppm_SelectCh')), 'Value');
%--- add 2011/06/25

%--- get samplingperiod info. from 1st data
datar=P3_gui_Research_1st('makeSSReal0',hs,false);
AD.samplingperiod = datar.AnaFiles{1}.samplingperiod;




%##########################################################################
% GUI <--> ExeData
%##########################################################################

%==========================================================================
function ExeData=MakeExeData(hs)
% Get Parameter's of 1st-Level-Analysis Execution
%==========================================================================

ExeData.AD=sub_Gather_Variables(hs);

ExeData.Tag={};
ExeData.Tgt={};
ExeData.Val={};

fn=fieldnames(hs);
for k=1:length(fn)
	if ~isempty(strfind(fn{k},mfilename))
		if strcmp(fn{k}(1:4),'edt_')
			ExeData.Tag{end+1} = fn{k};
			ExeData.Tgt{end+1} = 'String';
			ExeData.Val{end+1} = get(hs.(fn{k}),'String');
		elseif strcmp(fn{k}(1:4),'cbx_')
			ExeData.Tag{end+1} = fn{k};
			ExeData.Tgt{end+1} = 'Value';
			ExeData.Val{end+1} = get(hs.(fn{k}),'Value');			
		end
	end
end

apidata=getappdata(hs.figure1,mfilename);
for ii=1:length(apidata)
  try
    ExeData.([apidata{ii}.fcn num2str(ii)])=...
      feval(apidata{ii}.fcn,'MakeExeData',apidata{ii});
  catch
  end
end
ExeData.oldapidata=apidata; % for Normal-Group


%==========================================================================
function r=SetExeData(hs,ExeData)
% Set Parameter's of 1st-Level-Analysis Execution
%==========================================================================
r=0;
try
	for k=1:length(ExeData.Tag)
		set(hs.(ExeData.Tag{k}), ExeData.Tgt{k}, ExeData.Val{k});
	end		
	apidata=getappdata(hs.figure1,mfilename);
	for ii=1:length(apidata)
		try
			feval(apidata{ii}.fcn,'SetExeData',apidata{ii},...
				ExeData.([apidata{ii}.fcn num2str(ii)]));
		catch
		end
	end
	
catch
  r=1;
end



%##########################################################################
% Execution
%##########################################################################

%==========================================================================
function datar=execute(datar)
% Execute 1st-Level-Analysis
%
% Input : datar
%       datar.nfile    : number of files
%       datar.Anafiles : Analysis-File (with Recipe)
%       datar.ExeData  : Parameters (Set by function: MakeData)
% Output : datar
%       datar.Header     : Cell Header of Summarized Data
%       datar.Summarized : Cell Summarized Data
%==========================================================================
push_ExeData = datar.ExeData;

apidata=datar.ExeData.oldapidata;
for ii=1:length(apidata)
  try
    datar=feval(apidata{ii}.fcn,'execute',datar,...
      datar.ExeData.([apidata{ii}.fcn num2str(ii)]));
  catch
  end
end

datar=dofirst(datar);

%pop ExeData
datar.ExeData = push_ExeData;


%==========================================================================
function [datar logmsg] = dofirst(datar)
%==========================================================================
% Input : datar
%       datar.nfile    : number of files
%       datar.Anafiles : Analysis-File (with Recipe)
%       datar.ExeData  : Execution Parameter
%       datar.ExeData.ExecuteString ( Execute String)
%       datar.ExeData.ApplyFlag ( Check Flag or not)
% Output : datar
%       datar.Header     : Cell Header of Summarized Data
%       datar.Summarized : Cell Summarized Data
%==========================================================================

logmsg={'---- Summary Statistics Computation: Keio Regression analysis  version 0.0 ---'};
X=0;
P3_waitbar(X,logmsg,'Summary Statistics Computation'); 

%--- Get information of DataList...etc 
dfcn='DataDef2_Analysis'; %.. Data-Function
idkey = feval(dfcn,'getIdentifierKey');
dlist=feval(dfcn,'loadlist');
fns=fieldnames(dlist); %.. get fieldnames of Structure

A = datar.ExeData.AD;
tmphb = A.selectHbKind;
dur = 10 * (str2double(A.blkpre_sec) ...
    + str2double(A.taskDuration) + str2double(A.blkpost_sec));

%--- Get ID of selected file in Data List
% selectID = cell(1,datar.nfile);
% for tmpfile = 1 : datar.nfile;
%     selectID{tmpfile} = datar.AnaFiles{tmpfile}.ID_number;
% end
% uniID = unique(selectID);
% fileOrder = cell(1,length(uniID));
% for tmp = 1 : length(uniID)
%     fileOrder{1,tmp} = strmatch(uniID(tmp), selectID); %.. return indx
% end

%--- Initialize Summarized Data
%Header={'Data','Block','Channel','Kind','Stim',fns{:}};
% Header={'Reg_Number','Stim',fns{:}}; %.. Label for SummarizedKey
if A.selectedModelType == 1 || A.selectedModelType == 2
    Header={'beta','Reg_Number','Stim',fns{:}};
else
    Header={'beta','Reg_Number','Stim','maxcorr','delay',fns{:}}; %modifed on 2013/12/26
%    Header={'beta','Reg_Number','Stim','XC','delay',fns{:}};
end
SummarizedKey ={}; %.. Columns from 2nd to End
SummarizedData={}; %.. 1st Column
blkkey=[]; % SearchKey for Block (2010_2_RA06-1)
for ii=1:length(fns)
  if ~isstruct(dlist(1).(fns{ii})), continue; end
  tmp=dlist(1).(fns{ii});
  if ~isfield(tmp,'Type'), continue; end
  if any(strcmpi({'NumericB','TextB'},tmp.Type))
    blkkey(end+1)=ii;
  end
end

if isfield(datar.ExeData,'ApplyFlag') && datar.ExeData.ApplyFlag
  apflag=true;
  logmsg{end+1}=sprintf(' * Flag : Apply');
  P3_waitbar(X,logmsg(end));
else
  apflag=false;
end


%--- Make Model Data
if A.selectedModelType == 1
    %------ select Boxcar
    M = P3R2F_keioGLMtest('sub_MakeFIR_Model', A);
elseif A.selectedModelType == 2
    %------ select Model from File
    load(A.modelFileName);
    M = Model.HRF;
    if size(M,1) > size(M,2); M = M'; end;
else
    %------ select Optimal Model from File made in Stat.
    load(A.modelFileName);
    M = Model.HRF;
    if dur < length(M)
        M = M(1:dur);
    end
    if size(M,1) > size(M,2); M = M'; end;
    XC = Model.maxcorr; %.. <fileN, chN, hbN, stimN>
    Tau = Model.Tau; %.. <fileN, chN, hbN, stimN>
    MdimN = Model.selDim;
    timeReso = Model.timeReso;
end


%--- waitbar scale width
DX = 1/datar.nfile; 
% countfid = 0;

%---
for fid=1:datar.nfile
        
    myname=datar.AnaFiles{fid}.(idkey);
    X=(fid-1)*DX;
    logmsg{end+1}=sprintf('Start %s',myname);
    P3_waitbar(X,logmsg(end));

    %----- Evalute m-file == Execute 1st-Level-Analysis
    key.actdata.data=datar.AnaFiles{fid};
    fmd = datar.AnaFiles{fid}.data.filterdata; % Filter
    key.filterManage = fmd; %- fmd = Ana recipe (from 1 file(loop:fid))
    fname = feval(dfcn,'make_mfile',key);
    try
        [data,hdata] = scriptMeval(fname,'data', 'hdata');
    catch
        try delete(fname); catch;  end
        logmsg{end+1}=sprintf('[E] Evaluate Faild : %s',lasterr);
        P3_waitbar(X,logmsg(end));
        continue;
    end
    try 
       [p f]=fileparts(fname);
       clear(f); 
    catch
    end
    try delete(fname); catch;  end 
    
    %----- Regression Analysis : Main analysis
    stimkind = unique(hdata.stim(:,1));
    B = cell(1, size(data,2)); %.. Finally, the size o B(Beta) should be <regN, chN>
    regN_stim = zeros(length(stimkind), 2); %.. <stimN, 2(sp&ep)>
    % regressorstimNo.̑Ή֌W𖾂炩ɂĂ߂
    
	if A.selectedModelType == 1
        %------ select Boxcar
        for tmpch = 1 : size(data,2)
            R = []; %.. R should be finally the size of <regN, sampleN>
            for tmpstim = 1 : length(stimkind)
                regN_stim(tmpstim,1) = size(R,1)+1;
                R = [R; P3P_addRegressor_ChModel(hdata,A,M,tmpch,stimkind(tmpstim))];
                regN_stim(tmpstim,2) = size(R,1);
            end
            %---- Residual Regressor
            maxamp = max(max(abs(R)));
            N = str2num(A.sincosReg);
            R = [R; maxamp(1)*P3P_keioGLM_addRegressor_SinFluctuationNoise(hdata, N, tmpch)];
            R = [R; maxamp(1)*P3P_keioGLM_addRegressor_1(hdata, tmpch)];

            %... Regression Analysis
            B{1,tmpch} = regress(squeeze(data(:,tmpch,tmphb)), R');
        end

    elseif A.selectedModelType == 2
        %------ select Model from File
        for tmpch = 1 : size(data,2)
            R = [];
            for tmpstim = 1 : length(stimkind)
                regN_stim(tmpstim,1) = size(R,1)+1;
                R = [R; P3P_addRegressor_ChModel(hdata,A,M,tmpch,stimkind(tmpstim))];
                regN_stim(tmpstim,2) = size(R,1);
            end
            %---- Residual Regressor
            maxamp = max(max(abs(R)));
            N = str2num(A.sincosReg);
            R = [R; maxamp(1)*P3P_keioGLM_addRegressor_SinFluctuationNoise(hdata, N, tmpch)];
            R = [R; maxamp(1)*P3P_keioGLM_addRegressor_1(hdata, tmpch)];
            
            %... Regression Analysis
            B{1,tmpch} = regress(squeeze(data(:,tmpch,tmphb)), R');
        end
        
    else
        %------ select Optimal Model from File made in Stat.
        for tmpch = 1 : size(data,2)
            
            %---- file % ch No. check
            tmpf = MdimN{1,2}(1); %.. file Dimension Num is 1 (mean)
            tmpchl = MdimN{1,2}(2); %.. channel Dimension Num is 1 (mean)
            if tmpf > 1; tmpf = fid; end;
            if tmpchl > 1; tmpchl = tmpch; end;
            
            %---- make Regressors
            R = [];
            for tmpstim = 1 : length(stimkind)
%                 tmpTau = Tau(tmpf,tmpchl,1,tmpstim);
                tmpTau = Tau(tmpf,tmpchl,1,tmpstim);
                if Tau > 0
        %             M = [zeros(1,fix(Tau/timeReso)) M];
                    shift = zeros(1, fix(tmpTau/timeReso));
                    tmpM = [shift M(1:end-length(shift))];
                elseif Tau < 0
        %             M = M(-fix(Tau/timeReso):end);
                    shift = zeros(1, fix(-tmpTau/timeReso));
                    tmpM = [M(length(shift)+1:end) shift];
                else
                    tmpM = M;
                end
                               
                regN_stim(tmpstim,1) = size(R,1)+1;
                R = [R; P3P_addRegressor_ChModel(hdata,A,tmpM,tmpch,stimkind(tmpstim))];
                regN_stim(tmpstim,2) = size(R,1);
            end
            %---- Residual Regressor
            maxamp = max(max(abs(R)));
            N = str2num(A.sincosReg);
            R = [R; maxamp(1)*P3P_keioGLM_addRegressor_SinFluctuationNoise(hdata, N, tmpch)];
            R = [R; maxamp(1)*P3P_keioGLM_addRegressor_1(hdata, tmpch)];
    
            %... Regression Analysis
            B{1,tmpch} = regress(squeeze(data(:,tmpch,tmphb)), R');
        end
        
    end
 
    %[B] = P3_keio_sub_MakeSSData(hdata);
    
    clear hdata data;
	
    
	%----- Make SS-data
	tmps=dlist(strcmp({dlist.(idkey)}, myname));%- SummarizedKey (SearchKey)
    tmps=struct2cell(tmps);
    try
        for tmpstim = 1 : length(stimkind)
            sp = regN_stim(tmpstim,1);
            ep = regN_stim(tmpstim,2);
            tmpB = [];
            for tmpch = 1 : size(B,2)
                tmpB = [tmpB  B{1,tmpch}(sp:ep)];
			end
            for reg = 1 : size(tmpB,1)
                
                %----------------------------------------------
				if A.selectedModelType == 1 || A.selectedModelType == 2
                    t={ reg, sprintf('Stim%d', stimkind(tmpstim)) };
                else
                    tmpf = MdimN{1,2}(1); %.. file Dimension Num is 1 (mean)
                    if tmpf > 1; tmpf = fid; end;
                    tmpXC = squeeze(XC(tmpf,:,1,tmpstim));
                    tmpTau = squeeze(Tau(tmpf,:,1,tmpstim));
                    t={ reg, sprintf('Stim%d', stimkind(tmpstim)), ...
                         tmpXC, tmpTau}; %corrected on 2013/12/26
%                        num2cell(tmpXC), num2cell(tmpTau)};
                end
                
                tmps0 = sub_Check_BlockKeys(tmps, blkkey);
               
                t={num2cell(tmpB(reg,:)) t{:} tmps0{:}};
				%- tips: by using cell (size:{1 channel}), data will be understood as channel data.
								
                SummarizedKey(end+1,:)=t;
                SummarizedData(end+1,:)={tmpB(reg,:),false(1,size(tmpB,2))};
                %-----------------------------------------------
            end
        end
    catch
        logmsg{end+1}=sprintf('[E] Make Summarized Data Error: %s',lasterr);
        P3_waitbar(X,logmsg(end));
        continue;
    end

    logmsg{end+1}=sprintf('** OK **');
    P3_waitbar(X+DX,logmsg(end));
end
%----- END Make SS-data

%----- Save SS-data
datar.Header=Header;
datar.SummarizedKey = SummarizedKey;
datar.SummarizedData = SummarizedData;
%----- END Save SS-data

%uiwait(msgbox(logmsg,'Evaluate Logs','none','modal'));
P3_waitbar(1,{'Done','----------------------'});


%==========================================================================
function hdata=sub_ReRegressionAnalysis(data,hdata, Regressor, NameTag)
%==========================================================================
[B] = P3_keio_sub_MakeSSData(hdata);% B (3D or 2D) [regressor num, ch, stim kind( can take 1 )]

%- average Beta 
%B=squeeze(nanmean(B,2)); % B (2D or 1D)[regressor num, stim kind( can take 1 )] 
B=squeeze(nan_fcn('mean',B,2));
%-

origRG = hdata.Results.Regressor;
rgKind = fieldnames(hdata.Results.RegressAna);
for k=1:length(rgKind)
	newRG = sub_calcRegressor(origRG, B(:,k));
end
tmp = hdata.Results.Re;


%==========================================================================
function rg=sub_calcRegressor(regrStruct, B)
%==========================================================================
fns=fieldnames(regrStruct);
tmp=[];
for k=1:length(fns)
	tmp(k)=str2num(fns{k}(end));
end
stimKind = unique(tmp);

if mod(length(fns),length(stimKind))~=0 %- check
	error('');
end

%- calc regress
rg = regrStruct.(fns{1})*0;

if length(B) ~= length(fns)
	warndlg('???');
	rg=nan;
	return;
end

for k=1;length(fns)
	rg=rg+regrStruct.(fns{k})*B(k);
end


%############### Sub routine ##############################################

%==========================================================================
function tmps0 = sub_Check_BlockKeys(tmps, blkkey)
%==========================================================================
tmps0=tmps;
for kk=blkkey(:)' % SearchKey for Block (2010_2_RA06-1)
	try
		tmps0{kk}=tmps{kk}.Data{blk};
	catch
		switch lower(tmps{kk}.Type)
			case {'numericb'}
				tmps0{kk}=NaN;
			case {'textb'}
				tmps0{kk}='';
			otherwise
				tmps0{kk}=[];
		end
	end
end



%==========================================================================
function sub_Update_DataParmGUIs(ARG)
%==========================================================================
D=get(ARG.D.hnd_MakeData,'UserData');
set(ARG.D.hndDataSelectListbox.(mytag('ppm_SelectCh')),'String',1:size(D.ResultMatrix,2));



%==========================================================================
function [hs, newHS, p] = sub_MakeGUI_Popupmenu(hs, Name, Def_String, Def_Value, Label_String, p, dp)
%==========================================================================
newHS=[];
[prop,prop_t] = sub_sub_Get_Base_Properties(hs.figure1);
lblDY = 11;

tag=mytag(['txt_' Name '_Label']);
hs.(tag)=uicontrol(hs.figure1,prop_t{:},'TAG',tag,...
  'String',Label_String,...
  'Position',[p(1) p(2) p(3) lblDY]);
newHS(end+1)=hs.(tag);

tag=mytag(['ppm_' Name]);
hs.(tag)=uicontrol(hs.figure1,prop{:},'TAG',tag,...
  'style','popupmenu','BackgroundColor',[1 1 1],...
  'HorizontalAlignment','left',...
  'String',Def_String,'Value', Def_Value(1),...
  'Position',[p(1) p(2)-p(4)+lblDY p(3) p(4)-lblDY]);
newHS(end+1)=hs.(tag);
p(1) = p(1)+dp(1); p(2) = p(2)-dp(2);


%==========================================================================
function [hs, newHS, p] = sub_MakeGUI_Editbox(hs, Name, Def_String, Label_String, p, dp)
%==========================================================================
newHS=[];
[prop,prop_t] = sub_sub_Get_Base_Properties(hs.figure1);
lblDY=11;

tag=mytag(['edt_' Name]);
hs.(tag)=uicontrol(hs.figure1,prop{:},'TAG',tag,...
  'style','edit','BackgroundColor',[1 1 1],...
  'HorizontalAlignment','left',...
  'String',Def_String,...
  'Position',[p(1) p(2)-p(4)+lblDY p(3) p(4)-lblDY]);
newHS(end+1)=hs.(tag);

tag=mytag(['txt_' Name '_Label']);
hs.(tag)=uicontrol(hs.figure1,prop_t{:},'TAG',tag,...
  'String',Label_String,...
  'Position',[p(1) p(2) p(3) lblDY]);
newHS(end+1)=hs.(tag);
p(1) = p(1)+dp(1); p(2) = p(2)-dp(2);


%==========================================================================
function [hs, newHS, p] = sub_MakeGUI_Listbox(hs, Name, Def_String, Def_Value, Label_String, p, dp)
%==========================================================================
newHS=[];
[prop,prop_t] = sub_sub_Get_Base_Properties(hs.figure1);
lblDY = 11;

tag=mytag(['txt_' Name '_Label']);
hs.(tag)=uicontrol(hs.figure1,prop_t{:},'TAG',tag,...
  'String',Label_String,...
  'Position',[p(1) p(2) p(3) lblDY]);
newHS(end+1)=hs.(tag);

tag=mytag(['lbx_' Name]);
hs.(tag)=uicontrol(hs.figure1,prop{:},'TAG',tag,...
  'style','listbox','BackgroundColor',[1 1 1],...
  'HorizontalAlignment','left','max',100,'ListboxTop',1,...
  'String', Def_String, 'Value', Def_Value,...
  'Position',[p(1) p(2)-p(4)+lblDY p(3) p(4)-lblDY]);
set(hs.(tag),'Listboxtop',1);
newHS(end+1)=hs.(tag);
p(1) = p(1)+dp(1); p(2) = p(2)-dp(2);


%==========================================================================
function [hs, newHS, p] = sub_MakeGUI_Checkbox(hs, Name, Def_Value, Label_String, p,dp)
%==========================================================================
newHS=[];
[dummy,prop_t] = sub_sub_Get_Base_Properties(hs.figure1);

tag=mytag(['cbx_' Name]);
hs.(tag)=uicontrol(hs.figure1,prop_t{3:end},'TAG',tag,...
  'style','checkbox','String',Label_String,'Value',Def_Value,...
  'Position',p);
newHS(end+1)=hs.(tag);
p(1) = p(1)+dp(1); p(2) = p(2)-dp(2);


%==========================================================================
function [hs, newHS, p] = sub_MakeGUI_Button(hs, Name, CallbackString, Label_String, p,dp)
%==========================================================================
newHS=[];
[dummy,prop_t] = sub_sub_Get_Base_Properties(hs.figure1);

tag=mytag(['cbx_' Name]);
hs.(tag)=uicontrol(hs.figure1,prop_t{3:end},'TAG',tag,...
  'style','pushbutton','String',Label_String,...
	'Callback', CallbackString, 'Position',p);
newHS(end+1)=hs.(tag);
p(1) = p(1)+dp(1); p(2) = p(2)-dp(2);


%==========================================================================
function [hs, newHS] = sub_MakeGUI_Checkbox2(hs, Name, Def_Value, Label_String, p)
%==========================================================================
newHS=[];
[dummy,prop_t] = sub_sub_Get_Base_Properties(hs.figure1);

tag=mytag(['cbx_' Name]);
hs.(tag)=uicontrol(hs.figure1,prop_t{3:end},'TAG',tag,...
  'style','checkbox','String','','Value',Def_Value,...
  'Position',p);
newHS(end+1)=hs.(tag);

tag=mytag(['txt_' Name '_Label']);
cbstr = sprintf('hs=guidata(gcf);v=get(hs.%s,''value'');set(hs.%s,''value'',(v==0))',tag,tag);
hs.(tag)=uicontrol(hs.figure1,prop_t{:},'TAG',tag,'String',Label_String,...
  'Position',[p(1)+20 p(2) p(3) p(4)/2],'ButtonDownFcn',cbstr);
newHS(end+1)=hs.(tag);


%==========================================================================
function [prop,prop_t,c]=sub_sub_Get_Base_Properties(figH)
%==========================================================================
c=get(figH,'Color');
prop={'Units','pixels','Visible','off'};
prop_t={prop{:},'style','text','HorizontalAlignment','left','BackgroundColor',c};

